/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smbatvsign.c,v 1.2 2005/01/31 21:25:56 ca Exp $")
#include "sm/assert.h"
#include "sm/error.h"
#include "sm/time.h"
#include "sm/str.h"
#include "sm/sha.h"
#include "sm/batv.h"
#include "sm/batvcnf.h"

#if SM_USE_BATV
/*
**  SM_BATV_SIGN -- create a BATV signature
**
**	Parameters:
**		address -- address to sign
**		days -- days ("timestamp") to use for signature
**		key -- BATV key to use
**		batv_cnf -- BATV configuration
**		batv_sig -- BATV signature (output)
**
**	Returns:
**		usual return code
*/

sm_ret_T
sm_batv_sign(const char *address, uint days, uint key, batv_cnf_P batv_cnf, batv_P batv_sig)
{
	sm_ret_T ret;
	size_t l, l0, l1;
	sm_str_P str;
	batv_key_P batv_key;
	unsigned char md[SHA_DIGEST_LENGTH];

	SM_REQUIRE(address != NULL);
	SM_REQUIRE(batv_cnf != NULL);
	SM_REQUIRE(batv_sig != NULL);

	ret = sm_batv_getkey(key, batv_cnf, &batv_key);
	if (sm_is_err(ret))
		return ret;
	SM_REQUIRE(batv_key != NULL);
	l0 = strlen(address);
	l1 = strlen(batv_key->batv_priv_key);

	/* address + K (key number) DDD (expiration date in days) + key */
	l = l0 + 1 + 3 + l1;
	SM_ASSERT(l0 < l);
	SM_ASSERT(l1 < l);
	SM_ASSERT(l < INT_MAX - 2);
	str = sm_str_new(NULL, l + 1, l + 2);
	if (str == NULL)
		return ENOMEM;
	l0 = sm_str_printf(str, "%s%x%03d%s"
			, address
			, key & 0x0f
			, (days % 1000)
			, batv_key->batv_priv_key
			);
	if (l0 < l)
	{
		ret = sm_error_perm(SM_EM_BATV, EINVAL); /* XXX */
		goto error;
	}

	SHA1(sm_str_data(str), sm_str_getlen(str), md);

	sm_snprintf(batv_sig, sizeof(batv_T), "%02x%02x%02x",
		md[0], md[1], md[2]);

	return SM_SUCCESS;

  error:
	SM_STR_FREE(str);
	return ret;
}
#endif /* SM_USE_BATV */
