/*
 * Copyright (c) 2004, 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 *
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: smerr2txt.c,v 1.15 2005/06/03 01:17:06 ca Exp $")

#include "sm/error.h"
#include "sm/dns.h"
#include "sm/bdb.h"
#include "sm/rfc2821.h"

struct e2s_S
{
	int	 e2s_code;
	char	*e2s_str;
};

typedef struct e2s_S	e2s_T, *e2s_P;

#if 0
/*
convert define line to struct entry:
:'a,.s/^#define \(SM[^ 	]*\).*/	{ \1,	"\1"	},/
*/
#endif /* 0 */

static e2s_T
e_modules[] =
{
	{ SM_EM_NONE,	"SM_EM_NONE"	},
	{ SM_EM_AQ,	"SM_EM_AQ"	},
	{ SM_EM_IQDB,	"SM_EM_IQDB"	},
	{ SM_EM_IBDB,	"SM_EM_IBDB"	},
	{ SM_EM_EDB,	"SM_EM_EDB"	},
	{ SM_EM_CDB,	"SM_EM_CDB"	},
	{ SM_EM_DA,	"SM_EM_DA"	},
	{ SM_EM_OCC,	"SM_EM_OCC"	},
	{ SM_EM_DNS,	"SM_EM_DNS"	},
	{ SM_EM_SMTPC,	"SM_EM_SMTPC"	},
	{ SM_EM_SMTPS,	"SM_EM_SMTPS"	},
	{ SM_EM_MAPS,	"SM_EM_MAPS"	},
	{ SM_EM_MAP,	"SM_EM_MAP"	},
	{ SM_EM_BDB,	"SM_EM_BDB"	},
	{ SM_EM_STTHRIO,	"SM_EM_STTHRIO"	},
	{ SM_EM_STTHR,	"SM_EM_STTHR"	},
	{ SM_EM_EVTHR,	"SM_EM_EVTHR"	},
	{ SM_EM_RECCOM,	"SM_EM_RECCOM"	},
	{ SM_EM_HT,	"SM_EM_HT"	},
	{ SM_EM_RSC,	"SM_EM_RSC"	},
	{ SM_EM_STR,	"SM_EM_STR"	},
	{ SM_EM_RCB,	"SM_EM_RCB"	},
	{ SM_EM_TLS,	"SM_EM_TLS"	},
	{ SM_EM_SASL,	"SM_EM_SASL"	},
	{ SM_EM_NET,	"SM_EM_NET"	},
	{ SM_EM_IO,	"SM_EM_IO"	},
	{ SM_EM_LOG,	"SM_EM_LOG"	},
	{ SM_EM_ADDR,	"SM_EM_ADDR"	},
	{ SM_EM_IP,	"SM_EM_IP"	},
	{ SM_EM_Q_Q2AR,	"SM_EM_Q_Q2AR"	},
	{ SM_EM_Q_AR2Q,	"SM_EM_Q_AR2Q"	},
	{ SM_EM_Q_Q2SS,	"SM_EM_Q_Q2SS"	},
	{ SM_EM_Q_SS2Q,	"SM_EM_Q_SS2Q"	},
	{ SM_EM_Q_Q2SC,	"SM_EM_Q_Q2SC"	},
	{ SM_EM_Q_SC2Q,	"SM_EM_Q_SC2Q"	},
	{ SM_EM_Q_RDIBDB,	"SM_EM_Q_RDIBDB"	},
	{ SM_EM_Q_IQDB,	"SM_EM_Q_IQDB"	},
	{ SM_EM_Q_IBDB,	"SM_EM_Q_IBDB"	},
	{ SM_EM_Q_EDBC,	"SM_EM_Q_EDBC"	},
	{ SM_EM_Q_DSN,	"SM_EM_Q_DSN"	},
	{ SM_EM_Q_START,	"SM_EM_Q_START"	},
	{ SM_EM_Q_CTL2Q,	"SM_EM_Q_CTL2Q"	},
	{ SM_EM_Q_CONF,	"SM_EM_Q_CONF"	},
	{ SM_EM_Q_CLEAN,	"SM_EM_Q_CLEAN"	},
	{ SM_EM_Q_SCHED,	"SM_EM_Q_SCHED"	},
	{ SM_EM_Q,	"SM_EM_Q"	},
	{ SM_EM_AR_Q2AR,	"SM_EM_AR_Q2AR"	},
	{ SM_EM_AR_AR2Q,	"SM_EM_AR_AR2Q"	},
	{ SM_EM_AR_DNSW,	"SM_EM_AR_DNSW"	},
	{ SM_EM_AR_DNSR,	"SM_EM_AR_DNSR"	},
	{ SM_EM_AR_DNSD,	"SM_EM_AR_DNSD"	},
	{ SM_EM_AR_WAIT,	"SM_EM_AR_WAIT"	},
	{ SM_EM_AR,	"SM_EM_AR"	},
	{ SM_EM_CONF,	"SM_EM_CONF"	},
	{ SM_EM_PMILTER,	"SM_EM_PMILTER"	},
	{ SM_EM_THR,	"SM_EM_THR"	},
	{ SM_EM_BATV,	"SM_EM_BATV"	},
	{ SM_EM_UTIL,	"SM_EM_UTIL"	},
	{ 0,			NULL		}
};

static e2s_T
e_error[] =
{
	{ SM_E_NOMORE,	"SM_E_NOMORE"	},
	{ SM_E_UNEXPECTED,	"SM_E_UNEXPECTED"	},
	{ SM_E_RANGE,	"SM_E_RANGE"	},
	{ SM_E_2BIG,	"SM_E_2BIG"	},
	{ SM_E_FULL,	"SM_E_FULL"	},
	{ SM_E_OVFLW_SC,	"SM_E_OVFLW_SC"	},
	{ SM_E_OVFLW_NS,	"SM_E_OVFLW_NS"	},
	{ SM_E_NOTFOUND,	"SM_E_NOTFOUND"	},
	{ SM_E_STARTFAIL,	"SM_E_STARTFAIL"	},
	{ SM_E_SYNTAX,	"SM_E_SYNTAX"	},
	{ SM_E_USAGE,	"SM_E_USAGE"	},
	{ SM_E_DONTSTART,	"SM_E_DONTSTART"	},
	{ SM_E_VER_MIX,	"SM_E_VER_MIX"	},
	{ SM_E_NO_AR,	"SM_E_NO_AR"	},
	{ SM_E_NO_DA,	"SM_E_NO_DA"	},
	{ SM_E_NO_DA_FREE,	"SM_E_NO_DA_FREE"	},
	{ SM_E_PR_V_MISM,	"SM_E_PR_V_MISM"	},
	{ SM_E_PR_ERR,	"SM_E_PR_ERR"	},
	{ SM_E_RCB2LONG,	"SM_E_RCB2LONG"	},
	{ SM_E_TEMPMAP,	"SM_E_TEMPMAP"	},
	{ SM_E_PERMMAP,	"SM_E_PERMMAP"	},
	{ SM_E_CLOSEDMAP,	"SM_E_CLOSEDMAP"	},
	{ SM_E_NOMAP,	"SM_E_NOMAP"	},
	{ SM_E_MXEMPTY,	"SM_E_MXEMPTY"	},
	{ SM_E_TTMYSELF,	"SM_E_TTMYSELF"	},
	{ SM_E_ALIASEXP,	"SM_E_ALIASEXP"	},
	{ SM_E_ALIAS_REC,	"SM_E_ALIAS_REC"	},
	{ SM_E_IP_MISM,	"SM_E_IP_MISM"	},
	{ SM_E_NOSIG,	"SM_E_NOSIG"	},
	{ SM_E_SIGEXP,	"SM_E_SIGEXP"	},
	{ SM_E_RSR_PRB,	"SM_E_RSR_PRB"	},
	{ SM_E_ILL_PIPE,	"SM_E_ILL_PIPE"	},
	{ SM_E_RD,	"SM_E_RD"	},
	{ SM_E_EOF,	"SM_E_EOF"	},
	{ SM_E_WR,	"SM_E_WR"	},
	{ SM_E_BARE_NL,	"SM_E_BARE_NL"	},

	{ DNSR_NOTFOUND,	"DNSR Not Found" },
	{ DNSR_NO_DATA,	"Valid name, no data record of requested type" },
	{ DNSR_TEMP,	"query failed, try again" },
	{ DNSR_PERM,	"query failed, don't retry" },
	{ DNSR_TIMEOUT,	"query timed out" },
	{ DNSR_RECURSE,	"recursion needed" },
	{ DNSR_MXINVALID,	"MX record invalid"	},
	{ DNSR_PTRINVALID,	"PTR record invalid"	},
	{ DNSR_CNINVALID,	"CNAME record invalid"	},

	{ R2821_ERR_LEFT,	"missing '<'" },
	{ R2821_ERR_RIGHT,	"missing '>'" },
	{ R2821_ERR_ROUTE,	"route address (but not allowed)" },
	{ R2821_ERR_HOST,	"domain part does not start with an atom" },
	{ R2821_ERR_FQDN,	"domain part is not a FQDN" },
	{ R2821_ERR_LOCAL,	"missing local part" },
	{ R2821_ERR_AT,	"missing '@'" },
	{ R2821_ERR_EMPTY,	"<>, but not allowed" },
	{ R2821_ERR_MBOX,	"completely empty address" },
	{ R2821_ERR_COLON,	"missing ':' (unused)" },
	{ R2821_ERR_DOTSTR,	"\"dotstr\" syntax is wrong: expected atom after '.'" },
	{ R2821_ERR_DOMAIN,	"domain part is empty" },
	{ R2821_ERR_DOT,	"expecting ':' (unused)" },
	{ R2821_ERR_TRAILDOT,	"domain has trailing '.'" },
	{ R2821_ERR_ATDOMAIN,	"'@' in route address missing" },
	{ R2821_ERR_COMMA,	"',' in route address missing" },
	{ R2821_ERR_DOMSYNTAX,	"domain part has wrong syntax" },
	{ R2821_ERR_MORE,	"tokens after end" },
	{ R2821_ERR_DEL_MISS,	"domain doesn't end in delimiter" },
	{ R2821_ERR_TOOLONG,	"address too long" },
	{ R2821_ERR_TRAILBACKSL,	"trailing backslash" },
	{ R2821_ERR_CLS_BRACKET,	"too many closing brackets" },
	{ R2821_ERR_CLS_PARENTH,	"too many closing parentheses" },
	{ R2821_ERR_PARENTHESIS,	"parenthesis" },
	{ R2821_ERR_MUSTQUOTE,	"character must be quoted but isn't" },

	{ SM_CNF_E_NEWLINE_IN_STRING,	"newline in string" },
	{ SM_CNF_E_EOF_IN_STRING,	"EOF in string" },
	{ SM_CNF_E_BAD_CHAR,	"unexpected character" },
	{ SM_CNF_E_NO_MEMORY,	"out of memory" },
	{ SM_CNF_E_HEX_EXPECTED,	"hexadecimal digit expected after \\x, \\u or \\U" },
	{ SM_CNF_E_CHAR_OVERFLOW,	"character constant too large" },
	{ SM_CNF_E_INVALID,	"invalid argument to configuration function" },
	{ SM_CNF_E_READ,	"error while reading configuration file" },
	{ SM_CNF_E_READ_OPEN,	"cannot open configuration file for reading" },
	{ SM_CNF_E_READ_CLOSE,	"error closing configuration file after reading" },
	{ SM_CNF_E_SYNTAX,	"syntax error" },
	{ SM_CNF_E_NOT_FOUND,	"not found" },
	{ SM_CNF_E_ALREADY,	"already exists" },
	{ SM_CNF_E_TYPE,	"invalid value" },
	{ SM_CNF_E_NUL_IN_STRING,	"ascii NUL character in string" },
	{ SM_CNF_E_TOO_MANY,	"too many elements in array" },

	{ 0,			NULL		}
};

/* XXX HACK XXX is there a DB function to do this? */

static e2s_T
e_bdb4_2[] =
{
	{ DB_DONOTINDEX,	"DB_DONOTINDEX" },
#ifdef DB_FILEOPEN
	{ DB_FILEOPEN,	"DB_FILEOPEN" },
#endif
	{ DB_KEYEMPTY,	"DB_KEYEMPTY" },
	{ DB_KEYEXIST,	"DB_KEYEXIST" },
	{ DB_LOCK_DEADLOCK,	"DB_LOCK_DEADLOCK" },
	{ DB_LOCK_NOTGRANTED,	"DB_LOCK_NOTGRANTED" },
	{ DB_NOSERVER,	"DB_NOSERVER" },
	{ DB_NOSERVER_HOME,	"DB_NOSERVER_HOME" },
	{ DB_NOSERVER_ID,	"DB_NOSERVER_ID" },
	{ DB_NOTFOUND,	"DB_NOTFOUND" },
	{ DB_OLD_VERSION,	"DB_OLD_VERSION" },
	{ DB_PAGE_NOTFOUND,	"DB_PAGE_NOTFOUND" },
	{ DB_REP_DUPMASTER,	"DB_REP_DUPMASTER" },
#ifdef DB_REP_HANDLE_DEAD
	{ DB_REP_HANDLE_DEAD,	"DB_REP_HANDLE_DEAD" },
#endif
	{ DB_REP_HOLDELECTION,	"DB_REP_HOLDELECTION" },
#ifdef DB_REP_ISPERM
	{ DB_REP_ISPERM,	"DB_REP_ISPERM" },
#endif
	{ DB_REP_NEWMASTER,	"DB_REP_NEWMASTER" },
	{ DB_REP_NEWSITE,	"DB_REP_NEWSITE" },
#ifdef DB_REP_NOTPERM
	{ DB_REP_NOTPERM,	"DB_REP_NOTPERM" },
#endif
#ifdef DB_REP_OUTDATED
	{ DB_REP_OUTDATED,	"DB_REP_OUTDATED" },
#endif
	{ DB_REP_UNAVAIL,	"DB_REP_UNAVAIL" },
	{ DB_RUNRECOVERY,	"DB_RUNRECOVERY" },
	{ DB_SECONDARY_BAD,	"DB_SECONDARY_BAD" },
#ifdef DB_VERIFY_BAD
	{ DB_VERIFY_BAD,	"DB_VERIFY_BAD" },
#endif
#if defined(DB_BUFFER_SMALL) && ENOMEM != DB_BUFFER_SMALL
	{ DB_BUFFER_SMALL,	"DB_BUFFER_SMALL" },
#endif
	{ 0,			NULL		}
};

#if 0
static e2s_T
e_dns[] =
{
	{ DNSR_NOTFOUND,	"DNSR Not Found" },
	{ DNSR_NO_DATA,	"Valid name, no data record of requested type" },
	{ DNSR_TEMP,	"query failed, try again" },
	{ DNSR_PERM,	"query failed, don't retry" },
	{ DNSR_TIMEOUT,	"query timed out" },
	{ DNSR_RECURSE,	"recursion needed" },
	{ DNSR_MXINVALID,	"MX record invalid"	},
	{ DNSR_PTRINVALID,	"PTR record invalid"	},
	{ DNSR_CNINVALID,	"CNAME record invalid"	},
	{ 0,			NULL		}
};
#endif /* 0 */

static char *
finde(int ecode, e2s_P table)
{
	int i;

	i = 0;
	while (table[i].e2s_code != 0 || table[i].e2s_str != NULL)
	{
		if (table[i].e2s_code == ecode)
			return table[i].e2s_str;
		if (sm_error_value(table[i].e2s_code) == sm_error_value(ecode))
			return table[i].e2s_str;
		++i;
	}
	return "Unknown";
}

char *
smmod2txt(sm_error_T err)
{
	char *errtxt;

	errtxt = finde(sm_error_module(err), e_modules);
	return errtxt;
}

char *
smerr2txt(sm_error_T err)
{
	int errornumber, eno;
	char *errtxt;

	eno = errornumber = sm_error_value(err);
	errtxt = NULL;

#if HAVE_SYS_NERR
	if (errornumber > 0 && errornumber < sys_nerr)
#else
	if (errornumber > 0 && errornumber < SM_ERR_BASE)
#endif
		errtxt = strerror(errornumber);
	else if (SM_IS_BDB_ERR(err))
#if 0
		/*
		**  If this is used then every application needs to link
		**  against libdb.
		*/

		errtxt = db_strerror(eno);
#else
		errtxt = finde(eno, e_bdb4_2);
#endif
#if 0
	else if (errornumber >= DNS_ERR_BASE && errornumber < R2821_BASE)
		errtxt = finde(err, e_dns);
#endif /* 0 */
	else
		errtxt = finde(errornumber, e_error);
	return errtxt;
}
