/*
 * Copyright (c) 2002-2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: qm_qss_ctx.c,v 1.28 2005/03/15 19:56:10 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "sm/io.h"
#include "sm/rcb.h"
#include "sm/qmgr.h"
#include "sm/qmgr-int.h"
#include "sm/rcbcomm.h"
#include "qmgr.h"

/*
**  QSS_CTX_NEW -- Allocate a new QMGR - SMTPS context
**
**	Parameters:
**		qmgr_ctx -- QMGR context
**		pqss_ctx -- QMGR/SMTPS context (output)
**
**	Returns:
**		usual sm_error code
**
**	Last code review: 2003-10-17 15:51:17
*/

sm_ret_T
qss_ctx_new(qmgr_ctx_P qmgr_ctx, qss_ctx_P *pqss_ctx)
{
	qss_ctx_P qss_ctx;
	sm_ret_T ret;

	SM_IS_QMGR_CTX(qmgr_ctx);
	SM_REQUIRE(pqss_ctx != NULL);
	qss_ctx = (qss_ctx_P) sm_zalloc(sizeof(*qss_ctx));
	if (qss_ctx == NULL)
		return sm_error_temp(SM_EM_Q_Q2SS, ENOMEM);
	qss_ctx->qss_qmgr_ctx = qmgr_ctx;
	ret = sm_rcbcom_open(&(qss_ctx->qss_com));
	if (sm_is_err(ret))
		goto error;
	qss_ctx->qss_status = QSS_ST_NONE;
	qss_ctx->qss_id = QSS_ID_NONE;
	qss_ctx->sm_magic = SM_QSS_CTX_MAGIC;
	*pqss_ctx = qss_ctx;
	return SM_SUCCESS;

  error:
	SM_FREE_SIZE(qss_ctx, sizeof(*qss_ctx));
	return ret;
}

/*
**  QSS_CTX_FREE -- Free a QMGR - SMTPS context
**
**	Parameters:
**		qss_ctx -- QMGR/SMTPS context
**
**	Returns:
**		usual sm_error code (always SM_SUCCESS)
**
**	Locking: qss_ctx must be under control of the caller.
**
**	Last code review: 2003-10-17 15:52:26, see below
*/

sm_ret_T
qss_ctx_free(qss_ctx_P qss_ctx)
{
	if (qss_ctx == NULL)
		return SM_SUCCESS;
	(void) sm_rcbcom_close(&(qss_ctx->qss_com));
	qss_ctx->sm_magic = SM_MAGIC_NULL;
	sm_free_size(qss_ctx, sizeof(*qss_ctx));
	return SM_SUCCESS;
}
