/*
 * Copyright (c) 2005 Sendmail, Inc. and its suppliers.
 *	All rights reserved.
 *
 * By using this file, you agree to the terms and conditions set
 * forth in the LICENSE file which can be found at the top level of
 * the sendmail distribution.
 */

#include "sm/generic.h"
SM_RCSID("@(#)$Id: init_ali_lfl.c,v 1.2 2005/06/09 20:06:51 ca Exp $")
#include "sm/error.h"
#include "sm/assert.h"
#include "smar.h"
#include "sm/smardef.h"

/*
**  SMAR_INIT_ALI_LFL -- initialize SMAR alias lookup flags
**	(after reading configuration)
**
**	Parameters:
**		smar_ctx -- SMAR context
**			smar_ctx->smar_cnf.smar_cnf_alias_fl must be set!
**
**	Returns:
**		usual sm_error code
**
**	Side Effect: sets smar_ctx->smar_alias_lfl
**
**	Can we do this simpler, e.g., a 1-1 mapping?
*/

typedef struct ali_map_fl_S	ali_map_fl_T;
struct ali_map_fl_S
{
	uint32_t amf_smarcnf_fl;
	uint32_t amf_map_fl;
};
static ali_map_fl_T
ali_map_fl[] =
{
	{ SMARCNF_FL_ALI_DETPLUS,	SMMAP_LFL_DETPLUS	},
	{ SMARCNF_FL_ALI_DETSTAR,	SMMAP_LFL_DETSTAR	},
	{ SMARCNF_FL_ALI_STAR,		SMMAP_LFL_STAR		},
	{ SMARCNF_FL_ALI_DOTSUBDOM,	SMMAP_LFL_DOTSUBDOM	},
	{ SMARCNF_FL_ALI_MREPL,		SMMAP_LFL_MCR_REPL	},
};

sm_ret_T
smar_init_ali_lfl(smar_ctx_P smar_ctx)
{
	uint u;

	SM_REQUIRE(smar_ctx != NULL);

	/* translate configuration flags into map lookup flags */
	smar_ctx->smar_alias_lfl = SMMAP_LFL_ALIAS;
	for (u = 0; u < SM_ARRAY_SIZE(ali_map_fl); u++)
	{
		uint32_t map_fl;

		map_fl = ali_map_fl[u].amf_map_fl;
		if (SM_IS_FLAG(smar_ctx->smar_cnf.smar_cnf_alias_fl,
				ali_map_fl[u].amf_smarcnf_fl))
			smar_ctx->smar_alias_lfl |= map_fl;
		else
			smar_ctx->smar_alias_lfl &= ~map_fl;
	}
	return SM_SUCCESS;
}
